@@ -12,7 +12,8 @@ function build_nextpnr {
     # We'd ideally use pypy3 for speed (as works locally), but the version
     # our CI Ubuntu provides doesn't like some of the typing stuff
     python3 ../himbaechel/uarch/example/example_arch_gen.py ./example.bba
-    ./bba/bbasm --l ./example.bba ./example.bin
+    mkdir -p share/himbaechel/example
+    ./bba/bbasm --l ./example.bba share/himbaechel/example/example.bin
     popd
 }
 
@@ -22,6 +23,6 @@ function run_tests {
 
 function run_archcheck {
     pushd build
-    ./nextpnr-himbaechel --uarch example --chipdb ./example.bin --test
+    ./nextpnr-himbaechel --device EXAMPLE --test
     popd
 }
