@@ -83,6 +83,7 @@ template <typename R> struct ArchAPI : BaseCtx
     virtual WireId getConflictingWireWire(WireId wire) const = 0;
     virtual NetInfo *getConflictingWireNet(WireId wire) const = 0;
     virtual DelayQuad getWireDelay(WireId wire) const = 0;
+    virtual IdString getWireConstantValue(WireId wire) const = 0;
     // Pip methods
     virtual typename R::AllPipsRangeT getPips() const = 0;
     virtual PipId getPipByName(IdStringList name) const = 0;
@@ -116,9 +117,9 @@ template <typename R> struct ArchAPI : BaseCtx
     virtual float getDelayNS(delay_t v) const = 0;
     virtual delay_t getDelayFromNS(float ns) const = 0;
     virtual uint32_t getDelayChecksum(delay_t v) const = 0;
-    virtual bool getBudgetOverride(const NetInfo *net_info, const PortRef &sink, delay_t &budget) const = 0;
     virtual delay_t estimateDelay(WireId src, WireId dst) const = 0;
     virtual BoundingBox getRouteBoundingBox(WireId src, WireId dst) const = 0;
+    virtual bool getArcDelayOverride(const NetInfo *net_info, const PortRef &sink, DelayQuad &delay) const = 0;
     // Decal methods
     virtual typename R::DecalGfxRangeT getDecalGraphics(DecalId decal) const = 0;
     virtual DecalXY getBelDecal(BelId bel) const = 0;
