@@ -128,6 +128,12 @@ delay_t Context::getNetinfoRouteDelay(const NetInfo *net_info, const PortRef &us
     if (src_wire == WireId())
         return 0;
 
+    DelayQuad quad_result;
+    if (getArcDelayOverride(net_info, user_info, quad_result)) {
+        // Arch overrides delay
+        return quad_result.maxDelay();
+    }
+
     delay_t max_delay = 0;
 
     for (auto dst_wire : getNetinfoSinkWires(net_info, user_info)) {
@@ -173,6 +179,11 @@ DelayQuad Context::getNetinfoRouteDelayQuad(const NetInfo *net_info, const PortR
 
     DelayQuad result(std::numeric_limits<delay_t>::max(), std::numeric_limits<delay_t>::lowest());
 
+    if (getArcDelayOverride(net_info, user_info, result)) {
+        // Arch overrides delay
+        return result;
+    }
+
     for (auto dst_wire : getNetinfoSinkWires(net_info, user_info)) {
         WireId cursor = dst_wire;
         DelayQuad delay{0};
@@ -225,13 +236,11 @@ uint32_t Context::checksum() const
         if (ni.driver.cell)
             x = xorshift32(x + xorshift32(ni.driver.cell->name.index));
         x = xorshift32(x + xorshift32(ni.driver.port.index));
-        x = xorshift32(x + xorshift32(getDelayChecksum(ni.driver.budget)));
 
         for (auto &u : ni.users) {
             if (u.cell)
                 x = xorshift32(x + xorshift32(u.cell->name.index));
             x = xorshift32(x + xorshift32(u.port.index));
-            x = xorshift32(x + xorshift32(getDelayChecksum(u.budget)));
         }
 
         uint32_t attr_x_sum = 0;
