@@ -100,7 +100,11 @@ struct Context : Arch, DeterministicRNG
     // --------------------------------------------------------------
 
     // provided by report.cc
-    void writeReport(std::ostream &out) const;
+    void writeJsonReport(std::ostream &out) const;
+
+    // provided by timing_log.cc
+    void log_timing_results(TimingResult &result, bool print_histogram, bool print_fmax, bool print_path,
+                            bool warn_on_failure);
     // --------------------------------------------------------------
 
     uint32_t checksum() const;
