@@ -25,6 +25,7 @@
 #include "json_frontend.h"
 #include "log.h"
 #include "nextpnr.h"
+#include "rust.h"
 
 #include <fstream>
 #include <memory>
@@ -239,8 +240,6 @@ PYBIND11_EMBEDDED_MODULE(MODULE_NAME, m)
                                                                                                             "cell");
     readonly_wrapper<PortRef, decltype(&PortRef::port), &PortRef::port, conv_to_str<IdString>>::def_wrap(pr_cls,
                                                                                                          "port");
-    readonly_wrapper<PortRef, decltype(&PortRef::budget), &PortRef::budget, pass_through<delay_t>>::def_wrap(pr_cls,
-                                                                                                             "budget");
 
     auto pm_cls = py::class_<ContextualWrapper<PipMap &>>(m, "PipMap");
     readwrite_wrapper<PipMap &, decltype(&PipMap::pip), &PipMap::pip, conv_to_str<PipId>,
@@ -250,6 +249,9 @@ PYBIND11_EMBEDDED_MODULE(MODULE_NAME, m)
 
     m.def("parse_json", parse_json_shim);
     m.def("load_design", load_design_shim, py::return_value_policy::take_ownership);
+#ifdef USE_RUST
+    m.def("example_printnets", example_printnets);
+#endif
 
     auto region_cls = py::class_<ContextualWrapper<Region &>>(m, "Region");
     readwrite_wrapper<Region &, decltype(&Region::name), &Region::name, conv_to_str<IdString>,
@@ -302,22 +304,14 @@ PYBIND11_EMBEDDED_MODULE(MODULE_NAME, m)
     arch_wrap_python(m);
 }
 
-#ifdef MAIN_EXECUTABLE
-static wchar_t *program;
-#endif
-
 void (*python_sighandler)(int) = nullptr;
 
 void init_python(const char *executable)
 {
 #ifdef MAIN_EXECUTABLE
-    program = Py_DecodeLocale(executable, NULL);
-    if (program == NULL) {
-        fprintf(stderr, "Fatal error: cannot decode executable filename\n");
-        exit(1);
-    }
-    Py_SetProgramName(program);
-    py::initialize_interpreter();
+    static const char *python_argv[1];
+    python_argv[0] = executable;
+    py::initialize_interpreter(true, 1, python_argv);
     py::module::import(TOSTRING(MODULE_NAME));
     PyRun_SimpleString("from " TOSTRING(MODULE_NAME) " import *");
     python_sighandler = signal(SIGINT, SIG_DFL);
@@ -328,7 +322,6 @@ void deinit_python()
 {
 #ifdef MAIN_EXECUTABLE
     py::finalize_interpreter();
-    PyMem_RawFree(program);
 #endif
 }
 
