@@ -211,8 +211,6 @@ class SAPlacer
             if ((placed_cells - constr_placed_cells) % 500 != 0)
                 log_info("  initial placement placed %d/%d cells\n", int(placed_cells - constr_placed_cells),
                          int(autoplaced.size()));
-            if (cfg.budgetBased && cfg.slack_redist_iter > 0)
-                assign_budget(ctx);
             ctx->yield();
             auto iplace_end = std::chrono::high_resolution_clock::now();
             log_info("Initial placement time %.02fs\n",
@@ -240,8 +238,7 @@ class SAPlacer
 
         // Invoke timing analysis to obtain criticalities
         tmg.setup_only = true;
-        if (!cfg.budgetBased)
-            tmg.setup();
+        tmg.setup();
 
         // Calculate costs after initial placement
         setup_costs();
@@ -368,18 +365,12 @@ class SAPlacer
                     // temp = post_legalise_temp;
                     // diameter = std::min<int>(M, diameter * post_legalise_dia_scale);
                     ctx->shuffle(autoplaced);
-
-                    // Legalisation is a big change so force a slack redistribution here
-                    if (cfg.slack_redist_iter > 0 && cfg.budgetBased)
-                        assign_budget(ctx, true /* quiet */);
                 }
                 require_legal = false;
-            } else if (cfg.budgetBased && cfg.slack_redist_iter > 0 && iter % cfg.slack_redist_iter == 0) {
-                assign_budget(ctx, true /* quiet */);
             }
 
             // Invoke timing analysis to obtain criticalities
-            if (!cfg.budgetBased && cfg.timing_driven)
+            if (cfg.timing_driven)
                 tmg.run();
             // Need to rebuild costs after criticalities change
             setup_costs();
@@ -425,73 +416,51 @@ class SAPlacer
     }
 
   private:
+    std::vector<BelId> all_bels;
     // Initial random placement
     void place_initial(CellInfo *cell)
     {
-        bool all_placed = false;
-        int iters = 25;
-        while (!all_placed) {
-            BelId best_bel = BelId();
-            uint64_t best_score = std::numeric_limits<uint64_t>::max(),
-                     best_ripup_score = std::numeric_limits<uint64_t>::max();
+        while (cell) {
             CellInfo *ripup_target = nullptr;
-            BelId ripup_bel = BelId();
             if (cell->bel != BelId()) {
                 ctx->unbindBel(cell->bel);
             }
-            IdString targetType = cell->type;
+            FastBels::FastBelsData *bel_data;
+            auto type_cnt = fast_bels.getBelsForCellType(cell->type, &bel_data);
 
-            auto proc_bel = [&](BelId bel) {
-                if (ctx->isValidBelForCellType(targetType, bel)) {
-                    if (ctx->checkBelAvail(bel)) {
-                        uint64_t score = ctx->rng64();
-                        if (score <= best_score) {
-                            best_score = score;
-                            best_bel = bel;
-                        }
-                    } else {
-                        uint64_t score = ctx->rng64();
-                        CellInfo *bound_cell = ctx->getBoundBelCell(bel);
-                        if (score <= best_ripup_score && bound_cell->belStrength < STRENGTH_STRONG) {
-                            best_ripup_score = score;
-                            ripup_target = bound_cell;
-                            ripup_bel = bel;
-                        }
-                    }
-                }
-            };
-
-            if (cell->region != nullptr && cell->region->constr_bels) {
-                for (auto bel : cell->region->bels) {
-                    proc_bel(bel);
-                }
-            } else {
-                for (auto bel : ctx->getBels()) {
-                    proc_bel(bel);
-                }
-            }
-
-            if (best_bel == BelId()) {
-                if (iters == 0 || ripup_bel == BelId())
-                    log_error("failed to place cell '%s' of type '%s'\n", cell->name.c_str(ctx), cell->type.c_str(ctx));
-                --iters;
-                ctx->unbindBel(ripup_target->bel);
-                best_bel = ripup_bel;
-            } else {
-                ripup_target = nullptr;
-                all_placed = true;
-            }
-            ctx->bindBel(best_bel, cell, STRENGTH_WEAK);
-
-            if (!ctx->isBelLocationValid(best_bel)) {
-                ctx->unbindBel(best_bel);
+            while (true) {
+                int nx = ctx->rng(max_x + 1), ny = ctx->rng(max_y + 1);
+                if (cfg.minBelsForGridPick >= 0 && type_cnt < cfg.minBelsForGridPick)
+                    nx = ny = 0;
+                if (nx >= int(bel_data->size()))
+                    continue;
+                if (ny >= int(bel_data->at(nx).size()))
+                    continue;
+                const auto &fb = bel_data->at(nx).at(ny);
+                if (fb.size() == 0)
+                    continue;
+                BelId bel = fb.at(ctx->rng(int(fb.size())));
+                if (cell->region && cell->region->constr_bels && !cell->region->bels.count(bel))
+                    continue;
+                if (!ctx->isValidBelForCellType(cell->type, bel))
+                    continue;
+                ripup_target = ctx->getBoundBelCell(bel);
                 if (ripup_target != nullptr) {
-                    ctx->bindBel(best_bel, ripup_target, STRENGTH_WEAK);
+                    if (ripup_target->belStrength > STRENGTH_STRONG)
+                        continue;
+                    ctx->unbindBel(bel);
+                } else if (!ctx->checkBelAvail(bel)) {
+                    continue;
                 }
-                all_placed = false;
-                continue;
+                ctx->bindBel(bel, cell, STRENGTH_WEAK);
+                if (!ctx->isBelLocationValid(bel)) {
+                    ctx->unbindBel(bel);
+                    if (ripup_target)
+                        ctx->bindBel(bel, ripup_target, STRENGTH_WEAK);
+                    continue;
+                }
+                break;
             }
-
             // Back annotate location
             cell->attrs[ctx->id("BEL")] = ctx->getBelName(cell->bel).str(ctx);
             cell = ripup_target;
@@ -870,14 +839,10 @@ class SAPlacer
             return 0;
         if (ctx->getPortTimingClass(net->driver.cell, net->driver.port, cc) == TMG_IGNORE)
             return 0;
-        if (cfg.budgetBased) {
-            double delay = ctx->getDelayNS(ctx->predictArcDelay(net, user));
-            return std::min(10.0, std::exp(delay - ctx->getDelayNS(user.budget) / 10));
-        } else {
-            float crit = tmg.get_criticality(CellPortKey(user));
-            double delay = ctx->getDelayNS(ctx->predictArcDelay(net, user));
-            return delay * std::pow(crit, crit_exp);
-        }
+
+        float crit = tmg.get_criticality(CellPortKey(user));
+        double delay = ctx->getDelayNS(ctx->predictArcDelay(net, user));
+        return delay * std::pow(crit, crit_exp);
     }
 
     // Set up the cost maps
@@ -1267,7 +1232,6 @@ Placer1Cfg::Placer1Cfg(Context *ctx)
     constraintWeight = ctx->setting<float>("placer1/constraintWeight", 10);
     netShareWeight = ctx->setting<float>("placer1/netShareWeight", 0);
     minBelsForGridPick = ctx->setting<int>("placer1/minBelsForGridPick", 64);
-    budgetBased = ctx->setting<bool>("placer1/budgetBased", false);
     startTemp = ctx->setting<float>("placer1/startTemp", 1);
     timingFanoutThresh = std::numeric_limits<int>::max();
     timing_driven = ctx->setting<bool>("timing_driven");
