@@ -33,6 +33,7 @@
 #include <chrono>
 #include <deque>
 #include <fstream>
+#include <limits>
 #include <queue>
 #include <set>
 
@@ -73,6 +74,7 @@ struct Router2
 
     struct WireScore
     {
+        float delay;
         float cost;
         float togo_cost;
         float total() const { return cost + togo_cost; }
@@ -94,6 +96,7 @@ struct Router2
         // Visit data
         PipId pip_fwd, pip_bwd;
         bool visited_fwd = false, visited_bwd = false;
+        float cost_fwd = 0.0, cost_bwd = 0.0;
     };
 
     Context *ctx;
@@ -343,7 +346,8 @@ struct Router2
             return;
         WireId src = nets.at(net->udata).src_wire;
         WireId cursor = ad.sink_wire;
-        while (cursor != src) {
+        while (cursor != src &&
+               (net->constant_value == IdString() || ctx->getWireConstantValue(cursor) == net->constant_value)) {
             PipId pip = nd.wires.at(cursor).first;
             unbind_pip_internal(nd, user, cursor);
             cursor = ctx->getPipSrcWire(pip);
@@ -522,24 +526,19 @@ struct Router2
             flat_wires[w].pip_bwd = PipId();
             flat_wires[w].visited_fwd = false;
             flat_wires[w].visited_bwd = false;
+            flat_wires[w].cost_fwd = 0.0;
+            flat_wires[w].cost_bwd = 0.0;
         }
         t.dirty_wires.clear();
     }
 
     // These nets have very-high-fanout pips and special rules must be followed (only working backwards) to avoid
     // crippling perf
-    bool is_pseudo_const_net(const NetInfo *net)
-    {
-#ifdef ARCH_NEXUS
-        if (net->driver.cell != nullptr && net->driver.cell->type == id_VCC_DRV)
-            return true;
-#endif
-        return false;
-    }
+    bool is_dedi_const_net(const NetInfo *net) { return net->constant_value != IdString(); }
 
     void update_wire_by_loc(ThreadContext &t, NetInfo *net, store_index<PortRef> i, size_t phys_pin, bool is_mt)
     {
-        if (is_pseudo_const_net(net))
+        if (is_dedi_const_net(net))
             return;
         auto &nd = nets.at(net->udata);
         auto &ad = nd.arcs.at(i.idx()).at(phys_pin);
@@ -559,25 +558,33 @@ struct Router2
     }
 
     // Functions for marking wires as visited, and checking if they have already been visited
-    void set_visited_fwd(ThreadContext &t, int wire, PipId pip)
+    void set_visited_fwd(ThreadContext &t, int wire, PipId pip, float cost)
     {
         auto &wd = flat_wires.at(wire);
         if (!wd.visited_fwd && !wd.visited_bwd)
             t.dirty_wires.push_back(wire);
         wd.pip_fwd = pip;
         wd.visited_fwd = true;
+        wd.cost_fwd = cost;
     }
-    void set_visited_bwd(ThreadContext &t, int wire, PipId pip)
+    void set_visited_bwd(ThreadContext &t, int wire, PipId pip, float cost)
     {
         auto &wd = flat_wires.at(wire);
         if (!wd.visited_fwd && !wd.visited_bwd)
             t.dirty_wires.push_back(wire);
         wd.pip_bwd = pip;
         wd.visited_bwd = true;
+        wd.cost_bwd = cost;
     }
 
-    bool was_visited_fwd(int wire) { return flat_wires.at(wire).visited_fwd; }
-    bool was_visited_bwd(int wire) { return flat_wires.at(wire).visited_bwd; }
+    bool was_visited_fwd(int wire, float cost)
+    {
+        return flat_wires.at(wire).visited_fwd && flat_wires.at(wire).cost_fwd <= cost;
+    }
+    bool was_visited_bwd(int wire, float cost)
+    {
+        return flat_wires.at(wire).visited_bwd && flat_wires.at(wire).cost_bwd <= cost;
+    }
 
     float get_arc_crit(NetInfo *net, store_index<PortRef> i)
     {
@@ -600,20 +607,21 @@ struct Router2
         auto &nd = nets[net->udata];
         auto &ad = nd.arcs.at(i.idx()).at(phys_pin);
         auto &usr = net->users.at(i);
+        bool const_mode = is_dedi_const_net(net);
         ROUTE_LOG_DBG("Routing arc %d of net '%s' (%d, %d) -> (%d, %d)\n", i.idx(), ctx->nameOf(net), ad.bb.x0,
                       ad.bb.y0, ad.bb.x1, ad.bb.y1);
         WireId src_wire = ctx->getNetinfoSourceWire(net), dst_wire = ctx->getNetinfoSinkWire(net, usr, phys_pin);
-        if (src_wire == WireId())
+        if (src_wire == WireId() && !const_mode)
             ARC_LOG_ERR("No wire found for port %s on source cell %s.\n", ctx->nameOf(net->driver.port),
                         ctx->nameOf(net->driver.cell));
         if (dst_wire == WireId())
             ARC_LOG_ERR("No wire found for port %s on destination cell %s.\n", ctx->nameOf(usr.port),
                         ctx->nameOf(usr.cell));
-        int src_wire_idx = wire_to_idx.at(src_wire);
+        int src_wire_idx = const_mode ? -1 : wire_to_idx.at(src_wire);
         int dst_wire_idx = wire_to_idx.at(dst_wire);
         // Calculate a timing weight based on criticality
         float crit = get_arc_crit(net, i);
-        float crit_weight = (1.0f - std::pow(crit, 2));
+        float crit_weight = std::max<float>(0.05f, (1.0f - std::pow(crit, 2)));
         ROUTE_LOG_DBG("     crit=%.3f crit_weight=%.3f\n", crit, crit_weight);
         // Check if arc was already done _in this iteration_
         if (t.processed_sinks.count(dst_wire))
@@ -646,13 +654,14 @@ struct Router2
             ROUTE_LOG_DBG("src_wire = %s -> dst_wire = %s\n", ctx->nameOfWire(src_wire), ctx->nameOfWire(dst_wire));
 
             // Add 'forward' direction startpoints to queue
-            auto seed_queue_fwd = [&](WireId wire, float wire_cost = 0) {
+            auto seed_queue_fwd = [&](WireId wire) {
                 WireScore base_score;
-                base_score.cost = wire_cost;
+                base_score.delay = 0;
+                base_score.cost = 0;
                 int wire_idx = wire_to_idx.at(wire);
                 base_score.togo_cost = get_togo_cost(net, i, wire_idx, dst_wire, false, crit_weight);
                 t.fwd_queue.push(QueuedWire(wire_idx, base_score));
-                set_visited_fwd(t, wire_idx, PipId());
+                set_visited_fwd(t, wire_idx, PipId(), 0.0);
             };
             auto &dst_data = flat_wires.at(dst_wire_idx);
             // Look for nearby existing routing
@@ -669,20 +678,22 @@ struct Router2
 
             if (mode == 0 && t.fwd_queue.size() < 4)
                 continue;
-
-            if (mode == 1 && !is_pseudo_const_net(net)) {
-                // Seed forwards with the source wire, if less than 8 existing wires added
-                seed_queue_fwd(src_wire);
-            } else {
-                set_visited_fwd(t, src_wire_idx, PipId());
+            if (!const_mode) {
+                if (mode == 1) {
+                    // Seed forwards with the source wire, if less than 8 existing wires added
+                    seed_queue_fwd(src_wire);
+                } else {
+                    set_visited_fwd(t, src_wire_idx, PipId(), 0.0);
+                }
             }
             auto seed_queue_bwd = [&](WireId wire) {
                 WireScore base_score;
+                base_score.delay = 0;
                 base_score.cost = 0;
                 int wire_idx = wire_to_idx.at(wire);
                 base_score.togo_cost = get_togo_cost(net, i, wire_idx, src_wire, true, crit_weight);
                 t.bwd_queue.push(QueuedWire(wire_idx, base_score));
-                set_visited_bwd(t, wire_idx, PipId());
+                set_visited_bwd(t, wire_idx, PipId(), 0.0);
             };
 
             // Seed backwards with the dest wire
@@ -696,12 +707,12 @@ struct Router2
                                 : (!t.fwd_queue.empty() || !t.bwd_queue.empty())) &&
                    (!is_bb || iter < toexplore)) {
                 ++iter;
-                if (!t.fwd_queue.empty()) {
+                if (!t.fwd_queue.empty() && !const_mode) {
                     // Explore forwards
                     auto curr = t.fwd_queue.top();
                     t.fwd_queue.pop();
                     ++explored;
-                    if (was_visited_bwd(curr.wire)) {
+                    if (was_visited_bwd(curr.wire, std::numeric_limits<float>::max())) {
                         // Meet in the middle; done
                         midpoint_wire = curr.wire;
                         break;
@@ -715,7 +726,12 @@ struct Router2
                             continue;
                         WireId next = ctx->getPipDstWire(dh);
                         int next_idx = wire_to_idx.at(next);
-                        if (was_visited_fwd(next_idx)) {
+                        WireScore next_score;
+                        next_score.delay = curr.score.delay + cfg.get_base_cost(ctx, next, dh, crit_weight);
+                        next_score.cost = curr.score.cost + score_wire_for_arc(net, i, phys_pin, next, dh, crit_weight);
+                        next_score.togo_cost =
+                                cfg.estimate_weight * get_togo_cost(net, i, next_idx, dst_wire, false, crit_weight);
+                        if (was_visited_fwd(next_idx, next_score.delay)) {
                             // Don't expand the same node twice.
                             continue;
                         }
@@ -731,11 +747,7 @@ struct Router2
                             continue;
                         if (!thread_test_wire(t, nwd))
                             continue; // thread safety issue
-                        WireScore next_score;
-                        next_score.cost = curr.score.cost + score_wire_for_arc(net, i, phys_pin, next, dh, crit_weight);
-                        next_score.togo_cost =
-                                cfg.estimate_weight * get_togo_cost(net, i, next_idx, dst_wire, false, crit_weight);
-                        set_visited_fwd(t, next_idx, dh);
+                        set_visited_fwd(t, next_idx, dh, next_score.delay);
                         t.fwd_queue.push(QueuedWire(next_idx, next_score, t.rng.rng()));
                     }
                 }
@@ -744,12 +756,13 @@ struct Router2
                     auto curr = t.bwd_queue.top();
                     t.bwd_queue.pop();
                     ++explored;
-                    if (was_visited_fwd(curr.wire)) {
+                    auto &curr_data = flat_wires.at(curr.wire);
+                    if (was_visited_fwd(curr.wire, std::numeric_limits<float>::max()) ||
+                        (const_mode && ctx->getWireConstantValue(curr_data.w) == net->constant_value)) {
                         // Meet in the middle; done
                         midpoint_wire = curr.wire;
                         break;
                     }
-                    auto &curr_data = flat_wires.at(curr.wire);
                     // Don't allow the same wire to be bound to the same net with a different driving pip
                     PipId bound_pip;
                     auto fnd_wire = nd.wires.find(curr_data.w);
@@ -765,7 +778,14 @@ struct Router2
                             continue;
                         WireId next = ctx->getPipSrcWire(uh);
                         int next_idx = wire_to_idx.at(next);
-                        if (was_visited_bwd(next_idx)) {
+                        WireScore next_score;
+                        next_score.delay = curr.score.delay + cfg.get_base_cost(ctx, next, uh, crit_weight);
+                        next_score.cost = curr.score.cost + score_wire_for_arc(net, i, phys_pin, next, uh, crit_weight);
+                        next_score.togo_cost = const_mode
+                                                       ? 0
+                                                       : cfg.estimate_weight * get_togo_cost(net, i, next_idx, src_wire,
+                                                                                             true, crit_weight);
+                        if (was_visited_bwd(next_idx, next_score.delay)) {
                             // Don't expand the same node twice.
                             continue;
                         }
@@ -777,11 +797,7 @@ struct Router2
                             continue;
                         if (!thread_test_wire(t, nwd))
                             continue; // thread safety issue
-                        WireScore next_score;
-                        next_score.cost = curr.score.cost + score_wire_for_arc(net, i, phys_pin, next, uh, crit_weight);
-                        next_score.togo_cost =
-                                cfg.estimate_weight * get_togo_cost(net, i, next_idx, src_wire, true, crit_weight);
-                        set_visited_bwd(t, next_idx, uh);
+                        set_visited_bwd(t, next_idx, uh, next_score.delay);
                         t.bwd_queue.push(QueuedWire(next_idx, next_score, t.rng.rng()));
                     }
                 }
@@ -792,47 +808,51 @@ struct Router2
         ArcRouteResult result = ARC_SUCCESS;
         if (midpoint_wire != -1) {
             ROUTE_LOG_DBG("   Routed (explored %d wires): ", explored);
-            int cursor_bwd = midpoint_wire;
-            while (was_visited_fwd(cursor_bwd)) {
-                PipId pip = flat_wires.at(cursor_bwd).pip_fwd;
-                if (pip == PipId() && cursor_bwd != src_wire_idx)
-                    break;
-                bind_pip_internal(nd, i, cursor_bwd, pip);
-                if (ctx->debug && !is_mt) {
-                    auto &wd = flat_wires.at(cursor_bwd);
-                    ROUTE_LOG_DBG("      fwd wire: %s (curr %d hist %f share %d)\n", ctx->nameOfWire(wd.w),
-                                  wd.curr_cong - 1, wd.hist_cong_cost, nd.wires.at(wd.w).second);
+            if (const_mode) {
+                bind_pip_internal(nd, i, midpoint_wire, PipId());
+            } else {
+                int cursor_bwd = midpoint_wire;
+                while (was_visited_fwd(cursor_bwd, std::numeric_limits<float>::max())) {
+                    PipId pip = flat_wires.at(cursor_bwd).pip_fwd;
+                    if (pip == PipId() && cursor_bwd != src_wire_idx)
+                        break;
+                    bind_pip_internal(nd, i, cursor_bwd, pip);
+                    if (ctx->debug && !is_mt) {
+                        auto &wd = flat_wires.at(cursor_bwd);
+                        ROUTE_LOG_DBG("      fwd wire: %s (curr %d hist %f share %d)\n", ctx->nameOfWire(wd.w),
+                                      wd.curr_cong - 1, wd.hist_cong_cost, nd.wires.at(wd.w).second);
+                    }
+                    if (pip == PipId()) {
+                        break;
+                    }
+                    ROUTE_LOG_DBG("         fwd pip: %s (%d, %d)\n", ctx->nameOfPip(pip), ctx->getPipLocation(pip).x,
+                                  ctx->getPipLocation(pip).y);
+                    cursor_bwd = wire_to_idx.at(ctx->getPipSrcWire(pip));
                 }
-                if (pip == PipId()) {
-                    break;
-                }
-                ROUTE_LOG_DBG("         fwd pip: %s (%d, %d)\n", ctx->nameOfPip(pip), ctx->getPipLocation(pip).x,
-                              ctx->getPipLocation(pip).y);
-                cursor_bwd = wire_to_idx.at(ctx->getPipSrcWire(pip));
-            }
 
-            while (cursor_bwd != src_wire_idx) {
-                // Tack onto existing routing
-                WireId bwd_w = flat_wires.at(cursor_bwd).w;
-                if (!nd.wires.count(bwd_w))
-                    break;
-                auto &bound = nd.wires.at(bwd_w);
-                PipId pip = bound.first;
-                if (ctx->debug && !is_mt) {
-                    auto &wd = flat_wires.at(cursor_bwd);
-                    ROUTE_LOG_DBG("      ext wire: %s (curr %d hist %f share %d)\n", ctx->nameOfWire(wd.w),
-                                  wd.curr_cong - 1, wd.hist_cong_cost, bound.second);
+                while (cursor_bwd != src_wire_idx) {
+                    // Tack onto existing routing
+                    WireId bwd_w = flat_wires.at(cursor_bwd).w;
+                    if (!nd.wires.count(bwd_w))
+                        break;
+                    auto &bound = nd.wires.at(bwd_w);
+                    PipId pip = bound.first;
+                    if (ctx->debug && !is_mt) {
+                        auto &wd = flat_wires.at(cursor_bwd);
+                        ROUTE_LOG_DBG("      ext wire: %s (curr %d hist %f share %d)\n", ctx->nameOfWire(wd.w),
+                                      wd.curr_cong - 1, wd.hist_cong_cost, bound.second);
+                    }
+                    bind_pip_internal(nd, i, cursor_bwd, pip);
+                    if (pip == PipId())
+                        break;
+                    cursor_bwd = wire_to_idx.at(ctx->getPipSrcWire(pip));
                 }
-                bind_pip_internal(nd, i, cursor_bwd, pip);
-                if (pip == PipId())
-                    break;
-                cursor_bwd = wire_to_idx.at(ctx->getPipSrcWire(pip));
-            }
 
-            NPNR_ASSERT(cursor_bwd == src_wire_idx);
+                NPNR_ASSERT(cursor_bwd == src_wire_idx);
+            }
 
             int cursor_fwd = midpoint_wire;
-            while (was_visited_bwd(cursor_fwd)) {
+            while (was_visited_bwd(cursor_fwd, std::numeric_limits<float>::max())) {
                 PipId pip = flat_wires.at(cursor_fwd).pip_bwd;
                 if (pip == PipId()) {
                     break;
@@ -1012,7 +1032,7 @@ struct Router2
         auto &usr = net->users.at(usr_idx);
         WireId src = ctx->getNetinfoSourceWire(net);
         // Skip routes with no source
-        if (src == WireId())
+        if (src == WireId() && net->constant_value == IdString())
             return true;
         WireId dst = ctx->getNetinfoSinkWire(net, usr, phys_pin);
         if (dst == WireId())
@@ -1069,6 +1089,10 @@ struct Router2
                 break;
             }
             cursor = ctx->getPipSrcWire(p);
+            if (net->constant_value != IdString() && ctx->getWireConstantValue(cursor) == net->constant_value) {
+                src = cursor;
+                break;
+            }
         }
 
         if (success) {
@@ -1326,6 +1350,8 @@ struct Router2
     delay_t get_route_delay(int net, store_index<PortRef> usr_idx, int phys_idx)
     {
         auto &nd = nets.at(net);
+        if (nets_by_udata.at(net)->constant_value != IdString())
+            return 0;
         auto &ad = nd.arcs.at(usr_idx.idx()).at(phys_idx);
         WireId cursor = ad.sink_wire;
         if (cursor == WireId() || nd.src_wire == WireId())
@@ -1423,7 +1449,7 @@ struct Router2
             int tmgfail = 0;
             if (timing_driven)
                 tmg.run(false);
-            if (timing_driven_ripup && iter < 500) {
+            if (timing_driven_ripup && iter < 1500) {
                 for (size_t i = 0; i < nets_by_udata.size(); i++) {
                     NetInfo *ni = nets_by_udata.at(i);
                     for (auto usr : ni->users.enumerate()) {
@@ -1492,10 +1518,17 @@ Router2Cfg::Router2Cfg(Context *ctx)
     bb_margin_y = ctx->setting<int>("router2/bbMargin/y", 3);
     ipin_cost_adder = ctx->setting<float>("router2/ipinCostAdder", 0.0f);
     bias_cost_factor = ctx->setting<float>("router2/biasCostFactor", 0.25f);
-    init_curr_cong_weight = ctx->setting<float>("router2/initCurrCongWeight", 0.5f);
-    hist_cong_weight = ctx->setting<float>("router2/histCongWeight", 1.0f);
-    curr_cong_mult = ctx->setting<float>("router2/currCongWeightMult", 2.0f);
-    estimate_weight = ctx->setting<float>("router2/estimateWeight", 1.25f);
+    if (ctx->settings.count(ctx->id("router2/alt-weights"))) {
+        init_curr_cong_weight = ctx->setting<float>("router2/initCurrCongWeight", 5.0f);
+        hist_cong_weight = ctx->setting<float>("router2/histCongWeight", 0.5f);
+        curr_cong_mult = ctx->setting<float>("router2/currCongWeightMult", 0.0f);
+        estimate_weight = ctx->setting<float>("router2/estimateWeight", 1.0f);
+    } else {
+        init_curr_cong_weight = ctx->setting<float>("router2/initCurrCongWeight", 0.5f);
+        hist_cong_weight = ctx->setting<float>("router2/histCongWeight", 1.0f);
+        curr_cong_mult = ctx->setting<float>("router2/currCongWeightMult", 2.0f);
+        estimate_weight = ctx->setting<float>("router2/estimateWeight", 1.25f);
+    }
     perf_profile = ctx->setting<bool>("router2/perfProfile", false);
     if (ctx->settings.count(ctx->id("router2/heatmap")))
         heatmap = ctx->settings.at(ctx->id("router2/heatmap")).as_string();
