@@ -30,6 +30,7 @@
 #include "nextpnr.h"
 #include "placer1.h"
 #include "placer_heap.h"
+#include "placer_static.h"
 #include "router1.h"
 #include "router2.h"
 #include "timing.h"
@@ -582,23 +583,53 @@ delay_t Arch::predictDelay(BelId src_bel, IdString src_pin, BelId dst_bel, IdStr
            (3 + std::max(dx - 5, 0) + std::max(dy - 5, 0) + 2 * (std::min(dx, 5) + std::min(dy, 5)));
 }
 
-bool Arch::getBudgetOverride(const NetInfo *net_info, const PortRef &sink, delay_t &budget) const
-{
-    if (net_info->driver.port == id_FCO && sink.port == id_FCI) {
-        budget = 0;
-        return true;
-    } else if (sink.port.in(id_FXA, id_FXB)) {
-        budget = 0;
-        return true;
-    } else {
-        return false;
-    }
-}
-
 delay_t Arch::getRipupDelayPenalty() const { return 400; }
 
 // -----------------------------------------------------------------------
 
+namespace {
+void configure_static(Arch *arch, PlacerStaticCfg &cfg)
+{
+    {
+        cfg.cell_groups.emplace_back();
+        auto &comb = cfg.cell_groups.back();
+        comb.name = arch->id("COMB");
+        comb.bel_area[id_TRELLIS_COMB] = StaticRect(1.0f, 0.125f);
+        comb.bel_area[id_TRELLIS_RAMW] = StaticRect(0.0f, 0.0f);
+        comb.cell_area[id_TRELLIS_COMB] = StaticRect(1.0f, 0.125f);
+        comb.cell_area[id_TRELLIS_RAMW] = StaticRect(1.0f, 0.125f);
+        comb.zero_area_cells.insert(id_TRELLIS_RAMW);
+        comb.spacer_rect = StaticRect(1.0f, 0.125f);
+    }
+
+    {
+        cfg.cell_groups.emplace_back();
+        auto &comb = cfg.cell_groups.back();
+        comb.name = arch->id("FF");
+        comb.cell_area[id_TRELLIS_FF] = StaticRect(1.0f, 0.125f);
+        comb.bel_area[id_TRELLIS_FF] = StaticRect(1.0f, 0.125f);
+        comb.spacer_rect = StaticRect(1.0f, 0.125f);
+    }
+
+    {
+        cfg.cell_groups.emplace_back();
+        auto &comb = cfg.cell_groups.back();
+        comb.name = arch->id("RAM");
+        comb.cell_area[id_DP16KD] = StaticRect(2.0f, 1.0f);
+        comb.bel_area[id_DP16KD] = StaticRect(2.0f, 1.0f);
+        comb.spacer_rect = StaticRect(2.0f, 1.0f);
+    }
+    {
+        cfg.cell_groups.emplace_back();
+        auto &comb = cfg.cell_groups.back();
+        comb.name = arch->id("MUL");
+        comb.cell_area[id_MULT18X18D] = StaticRect(2.0f, 1.0f);
+        comb.bel_area[id_MULT18X18D] = StaticRect(2.0f, 1.0f);
+        comb.spacer_rect = StaticRect(2.0f, 1.0f);
+    }
+}
+} // namespace
+
 bool Arch::place()
 {
     std::string placer = str_or_default(settings, id_placer, defaultPlacer);
@@ -622,6 +653,11 @@ bool Arch::place()
 
         if (!placer_heap(getCtx(), cfg))
             return false;
+    } else if (placer == "static") {
+        PlacerStaticCfg cfg(getCtx());
+        configure_static(this, cfg);
+        if (!placer_static(getCtx(), cfg))
+            return false;
     } else if (placer == "sa") {
         if (!placer1(getCtx(), Placer1Cfg(getCtx())))
             return false;
@@ -653,7 +689,6 @@ bool Arch::route()
     setup_wire_locations();
     route_ecp5_globals(getCtx());
     assignArchInfo();
-    assign_budget(getCtx(), true);
 
     bool result;
     if (router == "router1") {
@@ -1240,7 +1275,7 @@ WireId Arch::get_bank_eclk(int bank, int eclk)
 
 const std::string Arch::defaultPlacer = "heap";
 
-const std::vector<std::string> Arch::availablePlacers = {"sa", "heap"};
+const std::vector<std::string> Arch::availablePlacers = {"sa", "heap", "static"};
 
 const std::string Arch::defaultRouter = "router1";
 const std::vector<std::string> Arch::availableRouters = {"router1", "router2"};
