@@ -328,6 +328,13 @@ void DesignWidget::newContext(Context *ctx)
                 WireInfo wi = ctx->wire_info(wire);
                 wireMap[std::pair<int, int>(wi.x, wi.y)].push_back(wire);
             }
+#endif
+#ifdef ARCH_HIMBAECHEL
+            for (const auto &wire : ctx->getWires()) {
+                Loc loc;
+                tile_xy(ctx->chip_info, wire.tile, loc.x, loc.y);
+                wireMap[std::pair<int, int>(loc.x, loc.y)].push_back(wire);
+            }
 #endif
             auto wireGetter = [](Context *ctx, WireId id) { return ctx->getWireName(id); };
             getTreeByElementType(ElementType::WIRE)
@@ -751,7 +758,6 @@ void DesignWidget::onSelectionChanged(int num, const QItemSelection &, const QIt
 
         QtProperty *driverItem = addSubGroup(topItem, "Driver");
         addProperty(driverItem, QVariant::String, "Port", net->driver.port.c_str(ctx));
-        addProperty(driverItem, QVariant::Double, "Budget", net->driver.budget);
         if (net->driver.cell)
             addProperty(driverItem, QVariant::String, "Cell", net->driver.cell->name.c_str(ctx), ElementType::CELL);
         else
@@ -762,7 +768,6 @@ void DesignWidget::onSelectionChanged(int num, const QItemSelection &, const QIt
             QtProperty *portItem = addSubGroup(usersItem, item.port.c_str(ctx));
 
             addProperty(portItem, QVariant::String, "Port", item.port.c_str(ctx));
-            addProperty(portItem, QVariant::Double, "Budget", item.budget);
             if (item.cell)
                 addProperty(portItem, QVariant::String, "Cell", item.cell->name.c_str(ctx), ElementType::CELL);
             else
