@@ -386,7 +386,7 @@ struct BelPinRange
 struct ArchArgs
 {
     std::string uarch;
-    std::string chipdb;
+    std::string chipdb_override;
     std::string device;
     dict<std::string, std::string> options;
 };
@@ -420,12 +420,17 @@ struct Arch : BaseArch<ArchRanges>
     Arch(ArchArgs args);
     ~Arch(){};
 
+    void load_chipdb(const std::string &path);
+    void set_speed_grade(const std::string &speed);
+    void set_package(const std::string &package);
+
     void late_init();
 
     // Database references
     boost::iostreams::mapped_file_source blob_file;
     const ChipInfoPOD *chip_info;
     const PackageInfoPOD *package_info = nullptr;
+    const SpeedGradePOD *speed_grade = nullptr;
 
     // Unlike Viaduct, we are not -generic based and therefore uarch must be non-nullptr
     std::unique_ptr<HimbaechelAPI> uarch;
@@ -483,6 +488,10 @@ struct Arch : BaseArch<ArchRanges>
     IdString getWireType(WireId wire) const override { return IdString(chip_wire_info(chip_info, wire).wire_type); }
     DelayQuad getWireDelay(WireId wire) const override { return DelayQuad(0); } // TODO
     BelPinRange getWireBelPins(WireId wire) const override { return BelPinRange(chip_info, get_tile_wire_range(wire)); }
+    IdString getWireConstantValue(WireId wire) const override
+    {
+        return IdString(chip_wire_info(chip_info, wire).const_value);
+    }
     WireRange getWires() const override { return WireRange(chip_info); }
     bool checkWireAvail(WireId wire) const override
     {
@@ -522,7 +531,36 @@ struct Arch : BaseArch<ArchRanges>
     {
         return normalise_wire(pip.tile, chip_pip_info(chip_info, pip).dst_wire);
     }
-    DelayQuad getPipDelay(PipId pip) const override { return DelayQuad(100); }
+    DelayQuad getPipDelay(PipId pip) const override
+    {
+        auto &pip_data = chip_pip_info(chip_info, pip);
+        auto pip_tmg = get_pip_timing(pip_data);
+        if (pip_tmg != nullptr) {
+            // TODO: multi corner analysis
+            WireId src = getPipSrcWire(pip);
+            uint64_t input_res = fast_pip_delays ? 0 : (drive_res.count(src) ? drive_res.at(src) : 0);
+            uint64_t input_cap = fast_pip_delays ? 0 : (load_cap.count(src) ? load_cap.at(src) : 0);
+            auto src_tmg = get_node_timing(src);
+            if (src_tmg != nullptr)
+                input_res += (src_tmg->res.slow_max / 2);
+            // Scale delay (fF * mOhm -> ps)
+            delay_t total_delay = (input_res * input_cap) / uint64_t(1e6);
+            total_delay += pip_tmg->int_delay.slow_max;
+
+            WireId dst = getPipDstWire(pip);
+            auto dst_tmg = get_node_timing(dst);
+            if (dst_tmg != nullptr) {
+                total_delay +=
+                        ((pip_tmg->out_res.slow_max + uint64_t(dst_tmg->res.slow_max) / 2) * dst_tmg->cap.slow_max) /
+                        uint64_t(1e6);
+            }
+
+            return DelayQuad(total_delay);
+        } else {
+            // Pip with no specified delay. Return a notional value so the router still has something to work with.
+            return DelayQuad(100);
+        }
+    }
     DownhillPipRange getPipsDownhill(WireId wire) const override
     {
         return DownhillPipRange(chip_info, get_tile_wire_range(wire));
@@ -547,11 +585,29 @@ struct Arch : BaseArch<ArchRanges>
     }
     void bindPip(PipId pip, NetInfo *net, PlaceStrength strength) override
     {
+        if (!fast_pip_delays) {
+            auto &pip_data = chip_pip_info(chip_info, pip);
+            auto pip_tmg = get_pip_timing(pip_data);
+            if (pip_tmg != nullptr) {
+                WireId src = getPipSrcWire(pip);
+                load_cap[src] += pip_tmg->in_cap.slow_max;
+                drive_res[getPipDstWire(pip)] =
+                        (((pip_tmg->flags & 1) || !drive_res.count(src)) ? 0 : drive_res.at(src)) +
+                        pip_tmg->out_res.slow_max;
+            }
+        }
         uarch->notifyPipChange(pip, net);
         BaseArch::bindPip(pip, net, strength);
     }
     void unbindPip(PipId pip) override
     {
+        if (!fast_pip_delays) {
+            auto &pip_data = chip_pip_info(chip_info, pip);
+            auto pip_tmg = get_pip_timing(pip_data);
+            if (pip_tmg != nullptr) {
+                load_cap[getPipSrcWire(pip)] -= pip_tmg->in_cap.slow_max;
+            }
+        }
         uarch->notifyPipChange(pip, nullptr);
         BaseArch::unbindPip(pip);
     }
@@ -568,10 +624,6 @@ struct Arch : BaseArch<ArchRanges>
     float getDelayNS(delay_t v) const override { return v * 0.001; }
     delay_t getDelayFromNS(float ns) const override { return delay_t(ns * 1000); }
     uint32_t getDelayChecksum(delay_t v) const override { return v; }
-    bool getBudgetOverride(const NetInfo *net_info, const PortRef &sink, delay_t &budget) const override
-    {
-        return false;
-    }
     BoundingBox getRouteBoundingBox(WireId src, WireId dst) const override
     {
         return uarch->getRouteBoundingBox(src, dst);
@@ -632,7 +684,20 @@ struct Arch : BaseArch<ArchRanges>
 
     // ------------------------------------------------
 
-    // clusters: BaseArch; for now
+    // Cluster methods
+    CellInfo *getClusterRootCell(ClusterId cluster) const override { return uarch->getClusterRootCell(cluster); }
+
+    BoundingBox getClusterBounds(ClusterId cluster) const override { return uarch->getClusterBounds(cluster); }
+
+    Loc getClusterOffset(const CellInfo *cell) const override { return uarch->getClusterOffset(cell); }
+
+    bool isClusterStrict(const CellInfo *cell) const override { return uarch->isClusterStrict(cell); }
+
+    bool getClusterPlacement(ClusterId cluster, BelId root_bel,
+                             std::vector<std::pair<CellInfo *, BelId>> &placement) const override
+    {
+        return uarch->getClusterPlacement(cluster, root_bel, placement);
+    }
     // ------------------------------------------------
 
     bool pack() override;
@@ -671,10 +736,71 @@ struct Arch : BaseArch<ArchRanges>
         }
     }
 
+    // -------------------------------------------------
+
+    const PipTimingPOD *get_pip_timing(const PipDataPOD &pip_data) const
+    {
+        int32_t idx = pip_data.timing_idx;
+        if (speed_grade && idx >= 0 && idx < speed_grade->pip_classes.ssize())
+            return &(speed_grade->pip_classes[idx]);
+        else
+            return nullptr;
+    }
+
+    const NodeTimingPOD *get_node_timing(WireId wire) const
+    {
+        int idx = -1;
+        if (!speed_grade)
+            return nullptr;
+        if (is_nodal_wire(chip_info, wire.tile, wire.index)) {
+            auto &shape = chip_node_shape(chip_info, wire.tile, wire.index);
+            idx = shape.timing_idx;
+        } else {
+            auto &wire_data = chip_wire_info(chip_info, wire);
+            idx = wire_data.timing_idx;
+        }
+        if (idx >= 0 && idx < speed_grade->node_classes.ssize())
+            return &(speed_grade->node_classes[idx]);
+        else
+            return nullptr;
+    }
+
+    // -------------------------------------------------
+
+    // Given cell type and variant, get the index inside the speed grade timing data
+    int get_cell_timing_idx(IdString type_variant) const;
+    // Return true and set delay if a comb path exists in a given cell timing index
+    bool lookup_cell_delay(int type_idx, IdString from_port, IdString to_port, DelayQuad &delay) const;
+    // Get setup and hold time and associated clock for a given cell timing index and signal
+    const RelSlice<CellPinRegArcPOD> *lookup_cell_seq_timings(int type_idx, IdString port) const;
+    // Attempt to look up port type based on timing database
+    TimingPortClass lookup_port_tmg_type(int type_idx, IdString port, PortType dir) const;
+
+    // -------------------------------------------------
+
+    bool getCellDelay(const CellInfo *cell, IdString fromPort, IdString toPort, DelayQuad &delay) const override;
+    // Get the port class, also setting clockInfoCount to the number of TimingClockingInfos associated with a port
+    TimingPortClass getPortTimingClass(const CellInfo *cell, IdString port, int &clockInfoCount) const override;
+    // Get the TimingClockingInfo of a port
+    TimingClockingInfo getPortClockingInfo(const CellInfo *cell, IdString port, int index) const override;
+
     // -------------------------------------------------
     void init_tiles();
+    void set_fast_pip_delays(bool fast_mode);
     std::vector<IdString> tile_name;
     dict<IdString, int> tile_name2idx;
+
+    // -------------------------------------------------
+    IdString get_tile_type(int tile) const;
+    const PadInfoPOD *get_package_pin(IdString pin) const;
+    const PadInfoPOD *get_bel_package_pin(BelId bel) const;
+    BelId get_package_pin_bel(IdString pin) const;
+
+    // Load capacitance and drive resistance for nodes
+    // TODO: does this `dict` hurt routing performance too much?
+    bool fast_pip_delays = false;
+    dict<WireId, uint64_t> drive_res;
+    dict<WireId, uint64_t> load_cap;
 };
 
 NEXTPNR_NAMESPACE_END
