@@ -27,8 +27,10 @@
 #include "nextpnr.h"
 #include "placer1.h"
 #include "placer_heap.h"
+#include "placer_static.h"
 #include "router1.h"
 #include "router2.h"
+#include "rust.h"
 #include "timing_opt.h"
 #include "util.h"
 NEXTPNR_NAMESPACE_BEGIN
@@ -639,47 +641,6 @@ std::vector<GroupId> Arch::getGroupGroups(GroupId group) const
 
 // -----------------------------------------------------------------------
 
-bool Arch::getBudgetOverride(const NetInfo *net_info, const PortRef &sink, delay_t &budget) const
-{
-    const auto &driver = net_info->driver;
-    if (driver.port == id_COUT) {
-        NPNR_ASSERT(sink.port.in(id_CIN, id_I3));
-        NPNR_ASSERT(driver.cell->constr_abs_z);
-        bool cin = sink.port == id_CIN;
-        bool same_y = driver.cell->constr_z < 7;
-        if (cin && same_y)
-            budget = 0;
-        else {
-            switch (args.type) {
-            case ArchArgs::HX8K:
-            case ArchArgs::HX4K:
-            case ArchArgs::HX1K:
-                budget = cin ? 190 : (same_y ? 260 : 560);
-                break;
-            case ArchArgs::LP384:
-            case ArchArgs::LP1K:
-            case ArchArgs::LP4K:
-            case ArchArgs::LP8K:
-                budget = cin ? 290 : (same_y ? 380 : 670);
-                break;
-            case ArchArgs::UP3K:
-            case ArchArgs::UP5K:
-            case ArchArgs::U1K:
-            case ArchArgs::U2K:
-            case ArchArgs::U4K:
-                budget = cin ? 560 : (same_y ? 660 : 1220);
-                break;
-            default:
-                log_error("Unsupported iCE40 chip type.\n");
-            }
-        }
-        return true;
-    }
-    return false;
-}
-
-// -----------------------------------------------------------------------
-
 bool Arch::place()
 {
     std::string placer = str_or_default(settings, id_placer, defaultPlacer);
@@ -691,6 +652,64 @@ bool Arch::place()
     } else if (placer == "sa") {
         if (!placer1(getCtx(), Placer1Cfg(getCtx())))
             return false;
+    } else if (placer == "static") {
+        PlacerStaticCfg cfg(getCtx());
+        cfg.logic_groups = 1;
+        {
+            cfg.cell_groups.emplace_back();
+            auto &comb = cfg.cell_groups.back();
+            comb.name = getCtx()->id("COMB");
+            comb.cell_area[id_ICESTORM_LC] = StaticRect(1.0f, 0.125f);
+            comb.bel_area[id_ICESTORM_LC] = StaticRect(1.0f, 0.125f);
+            comb.spacer_rect = StaticRect(1.0f, 0.125f);
+        }
+
+        {
+            cfg.cell_groups.emplace_back();
+            auto &comb = cfg.cell_groups.back();
+            comb.name = getCtx()->id("RAM");
+            comb.cell_area[id_ICESTORM_RAM] = StaticRect(1.0f, 2.0f);
+            comb.bel_area[id_ICESTORM_RAM] = StaticRect(1.0f, 2.0f);
+            comb.spacer_rect = StaticRect(1.0f, 2.0f);
+        }
+
+        {
+            cfg.cell_groups.emplace_back();
+            auto &comb = cfg.cell_groups.back();
+            comb.name = getCtx()->id("DSP");
+            comb.cell_area[id_ICESTORM_DSP] = StaticRect(0.9f, 5.0f);
+            comb.bel_area[id_ICESTORM_DSP] = StaticRect(0.9f, 5.0f);
+            comb.spacer_rect = StaticRect(0.9f, 5.0f);
+        }
+
+        {
+            cfg.cell_groups.emplace_back();
+            auto &comb = cfg.cell_groups.back();
+            comb.name = getCtx()->id("GB");
+            comb.cell_area[id_SB_GB] = StaticRect(0.5f, 0.5f);
+            comb.bel_area[id_SB_GB] = StaticRect(0.5f, 0.5f);
+            comb.spacer_rect = StaticRect(0.5f, 0.5f);
+        }
+
+        {
+            cfg.cell_groups.emplace_back();
+            auto &comb = cfg.cell_groups.back();
+            comb.name = getCtx()->id("WARMBOOT");
+            comb.cell_area[id_SB_WARMBOOT] = StaticRect(0.5f, 1.0f);
+            comb.bel_area[id_SB_WARMBOOT] = StaticRect(0.5f, 1.0f);
+            comb.spacer_rect = StaticRect(0.5f, 1.0f);
+        }
+
+        {
+            cfg.cell_groups.emplace_back();
+            auto &comb = cfg.cell_groups.back();
+            comb.name = getCtx()->id("IO");
+            comb.cell_area[id_SB_IO] = StaticRect(0.5f, 0.5f);
+            comb.bel_area[id_SB_IO] = StaticRect(0.5f, 0.5f);
+            comb.spacer_rect = StaticRect(0.5f, 0.5f);
+        }
+        if (!placer_static(getCtx(), cfg))
+            return false;
     } else {
         log_error("iCE40 architecture does not support placer '%s'\n", placer.c_str());
     }
@@ -1291,7 +1310,7 @@ BoundingBox Arch::getRouteBoundingBox(WireId src, WireId dst) const
 
 const std::string Arch::defaultPlacer = "heap";
 
-const std::vector<std::string> Arch::availablePlacers = {"sa", "heap"};
+const std::vector<std::string> Arch::availablePlacers = {"sa", "heap", "static"};
 
 const std::string Arch::defaultRouter = "router1";
 const std::vector<std::string> Arch::availableRouters = {"router1", "router2"};
