@@ -24,10 +24,10 @@ esac
 set -ex
 
 ${YOSYS:-yosys} -p "read_verilog ${1}.v
-                    synth_machxo2 -json ${1}.json
+                    synth_lattice -family xo2 -json ${1}.json
                     show -format png -prefix ${1}"
 ${NEXTPNR:-../../nextpnr-machxo2} $NEXTPNR_MODE --device LCMXO2-1200HC-4SG32C --json ${1}.json --write ${2}${1}.json
-${YOSYS:-yosys} -p "read_verilog -lib +/machxo2/cells_sim.v
+${YOSYS:-yosys} -p "read_verilog -lib +/lattice/cells_sim_xo2.v
                     read_json ${2}${1}.json
                     clean -purge
                     show -format png -prefix ${2}${1}
