@@ -48,6 +48,7 @@ struct ALMInfo
     std::array<BelId, 4> ff_bels;
 
     bool l6_mode = false;
+    bool carry_mode = false;
 
     // Which CLK/ENA and ACLR is chosen for each half
     std::array<int, 2> clk_ena_idx, aclr_idx;
@@ -288,6 +289,9 @@ struct Arch : BaseArch<ArchRanges>
     ArchArgs args;
     mistral::CycloneV *cyclonev;
 
+    // Mistral needs the bitstream configuring before it can use the simulator.
+    bool bitstream_configured = false;
+
     Arch(ArchArgs args);
     ArchArgs archArgs() const override { return args; }
 
@@ -431,8 +435,9 @@ struct Arch : BaseArch<ArchRanges>
                                        int &clockInfoCount) const override;                                // delay.cc
     TimingClockingInfo getPortClockingInfo(const CellInfo *cell, IdString port, int index) const override; // delay.cc
     bool getCellDelay(const CellInfo *cell, IdString fromPort, IdString toPort,
-                      DelayQuad &delay) const override; // delay.cc
-    DelayQuad getPipDelay(PipId pip) const override;    // delay.cc
+                      DelayQuad &delay) const override;                                                      // delay.cc
+    DelayQuad getPipDelay(PipId pip) const override;                                                         // delay.cc
+    bool getArcDelayOverride(const NetInfo *net_info, const PortRef &sink, DelayQuad &delay) const override; // delay.cc
 
     // -------------------------------------------------
 
@@ -503,6 +508,7 @@ struct Arch : BaseArch<ArchRanges>
     // -------------------------------------------------
 
     bool is_clkbuf_cell(IdString cell_type) const; // globals.cc
+    void route_globals();                          // globals.cc
 
     // -------------------------------------------------
 
