@@ -27,8 +27,8 @@ void Arch::create_gpio(int x, int y)
 {
     for (int z = 0; z < 4; z++) {
         // Notional pad wire
-        WireId pad = add_wire(x, y, id(stringf("PAD[%d]", z)));
-        BelId bel = add_bel(x, y, id(stringf("IO[%d]", z)), id_MISTRAL_IO);
+        WireId pad = add_wire(x, y, idf("PAD[%d]", z));
+        BelId bel = add_bel(x, y, idf("IO[%d]", z), id_MISTRAL_IO);
         add_bel_pin(bel, id_PAD, PORT_INOUT, pad);
         if (has_port(CycloneV::GPIO, x, y, z, CycloneV::DATAOUT, 0)) {
             // FIXME: is the port index of zero always correct?
