@@ -700,8 +700,6 @@ delay_t Arch::predictDelay(BelId src_bel, IdString src_pin, BelId dst_bel, IdStr
     return 100 * dist_x + 100 * dist_y + 250;
 }
 
-bool Arch::getBudgetOverride(const NetInfo *net_info, const PortRef &sink, delay_t &budget) const { return false; }
-
 BoundingBox Arch::getRouteBoundingBox(WireId src, WireId dst) const
 {
     BoundingBox bb;
@@ -811,8 +809,6 @@ float router2_base_cost(Context *ctx, WireId wire, PipId pip, float crit_weight)
 
 bool Arch::route()
 {
-    assign_budget(getCtx(), true);
-
     pre_routing();
 
     route_globals();
